import configparser
import json
from typing import Dict, Protocol

import toml
import yaml


class Loader(Protocol):
    def load(self, f) -> Dict:
        pass


class ConfigLoader:
    """A factory class for loading configuration files"""

    @staticmethod
    def load_config_file(filename: str) -> Dict:
        """Load a configuration file"""
        file_type = filename.split(".")[-1]
        with open(filename, "r") as f:
            loader = ConfigLoader._get_loader(file_type)
            return loader.load(f)

    @staticmethod
    def _get_loader(file_type: str):
        """Factory method to get a loader object"""
        loaders = {
            "yaml": YAMLLoader(),
            "ini": INILoader(),
            "json": JSONLoader(),
            "toml": TOMLLoader(),
        }
        try:
            return loaders[file_type]
        except KeyError:
            raise KeyError(f"Invalid file type: {file_type}")


class YAMLLoader:
    """A loader class for YAML configuration files"""

    def load(self, f: str) -> Dict:
        return yaml.safe_load(f)


class INILoader:
    """A loader class for INI configuration files"""

    def load(self, f: str) -> Dict:
        config = configparser.ConfigParser()
        config.read_file(f)
        return {section: dict(config[section]) for section in config.sections()}


class JSONLoader:
    """A loader class for JSON configuration files"""

    def load(self, f: str) -> Dict:
        return json.load(f)


class TOMLLoader:
    """A loader class for TOML configuration files"""

    def load(self, f: str) -> Dict:
        return toml.load(f)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['configgy']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'configparser>=5.3.0,<6.0.0', 'toml>=0.10.2,<0.11.0']

setup_kwargs = {
    'name': 'configgy',
    'version': '0.1.0',
    'description': 'Load popular config file formats into a python dict',
    'long_description': '# Configgy\n\nA subtle Python package for loading configuration files without having to specify the file type.\n\n## Installation\n\nYou can install the package from PyPI using pip:\n\n```bash\npip install configgy\n```\n\n## Usage\n\nHere\'s an example of how to use the package to load a configuration file:\n\n```python\nfrom configgy.loader import ConfigLoader\n\ndata = ConfigLoader().load_config_file("file.json")\n```\n\nThe `load_config` function takes a single argument, the path to the configuration file. It will automatically detect the file type and use the appropriate loader.\n\nCurrently, the package supports the following file types:\n\n- INI\n- YAML\n- TOML\n- JSON\n',
    'author': 'baniasbaabe',
    'author_email': 'banias@hotmail.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': '',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
